function skeletonvector = skeleton2vector(skeleton)
%SKELETON2VECTOR Convert graph skeleton(s) to vector(s)
%
%   VECTOR = SKELETON2VECTOR(SKELETON) converts skeleton SKELETON of a
%   graph to the column vector VECTOR. Only the lower triangular part of
%   SKELETON is used. If SKELETON is a sparse (full) matrix, VECTOR is
%   also a sparse (full) vector.
%
%   VECTORS = SKELETON2VECTOR(SKELETONS), where SKELETONS is a 1-by-N
%   (N-by-1) cell array of skeletons, converts each skeleton to a column
%   vector and returns them in the 1-by-N (N-by-1) cell array of column
%   vectors VECTORS.
%
%   Example:
%
%       skeleton = [0 1 2 4; 1 0 3 5; 2 3 0 6; 4 5 6 0]
%       vector = org.mensxmachina.commons.graph.skeleton2vector(skeleton)
%
%   See also:
%       ORG.MENSXMACHINA.COMMONS.GRAPH.VECTOR2SKELETON

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

if ~iscell(skeleton)
    
    skeletonvector = skeleton2vector_single(skeleton);
    
else
    
    skeletonvector = cell(size(skeleton));
    
    for i=1:length(skeleton)
        skeletonvector{i} = skeleton2vector_single(skeleton{i});
    end
    
end
    
end

function skeletonvector = skeleton2vector_single(skeleton)
%SKELETON2VECTOR_SINGLE Convert a single skeleton to vector

if issparse(skeleton)

    m = size(skeleton, 1);
    
    mchoose2 = (numel(skeleton) - m)/2;
    skeletonvector = spalloc(mchoose2, 1, nnz(skeleton)/2);
    
    [row col] = find(skeleton);
    
    for k=1:numel(row)
    
        if row(k) > col(k) % in lower triangular
        
            i = (col(k)-1)*(m - col(k)/2) + row(k) - col(k);

            skeletonvector(i) = skeleton(row(k), col(k));
            
        end

    end
    
else
    
    % convert skeleton matrix to a column vector
    skeletonvector = skeleton(:);

    % find indices of the lower triangular of the skeleton matrix
    % in the skeleton vector
    trilind = tril(true(size(skeleton)), -1);
    trilind = trilind(:)';

    % keep only elements of the vector corresponding to the lower triangular
    skeletonvector = skeletonvector(trilind);

end
    
end