function skeleton = vector2skeleton(vector)
%VECTOR2SKELETON Convert skeleton vector(s) to matrix(matrices)
%
%   SKELETON = VECTOR2SKELETON(VECTOR) converts the skeleton vector VECTOR
%   of a graph to the SKELETON matrix. If VECTOR is a sparse (full) vector,
%   SKELETON is also a sparse (full) matrix.
%
%   SKELETONS = VECTOR2SKELETON(VECTORS), where VECTORS is a 1-by-N
%   (N-by-1) cell array of skeleton vectors, converts all vectors to 
%   skeleton matrices and returns them in the 1-by-N (N-by-1) cell array of
%   skeleton matrices SKELETONS.
%
%   Example:
%
%       vector = [ 1 2 4 3 5 6 ]'
%       skeleton = org.mensxmachina.commons.graph.vector2skeleton(vector)
%
%   See also:
%       ORG.MENSXMACHINA.COMMONS.GRAPH.SKELETON2VECTOR

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

if ~iscell(vector)
    skeleton = vector2skeleton_single(vector);
else
    
    skeleton = cell(size(vector));
    
    for i=1:length(vector)
        skeleton{i} = vector2skeleton_single(vector{i});
    end
    
end

end

function skeleton = vector2skeleton_single(vector)
%VECTOR2SKELETON_SINGLE Convert single skeleton vector to matrix

% calculate # of variables
m = (1 + sqrt(1 + 8*length(vector)))/2;

if issparse(vector)
    
    skeleton = spalloc(m, m, nnz(vector)*2);
    
    j = 1; % current column
    
    % # of elements in the lower triangular, up to the previous column
    s_prev = 0;
    s = m - 1;
    
    for k = find(vector)' % for each (ordered) nonzero element
        
        % find j
        
        for j=j:m-1
            
            if k > s_prev && k <= s
            
                i = k - (j - 1)*(m - j/2) + j;
                
                skeleton(i, j) = vector(k);
                skeleton(j, i) = vector(k);
                
                break;
                
            end
            
            s_prev = s;
            s = s_prev + (m - j - 1);
            
        end
        
    end
    
else

    % initialize the skeleton matrix
    skeleton = zeros(m, m);
    
    % find indices of the lower triangular of the skeleton matrix
    % in the skeleton vector
    trilind = tril(true(size(skeleton)), -1);
    trilind = trilind(:)';

    skeleton(trilind) = vector;
    
    % make skeleton matrix symmetric
    skeleton = skeleton + skeleton';

end
    
end