function [auc tpr fpr thresholds] = auc(targets, outputs)
%AUC Area under the ROC curve (AUC)
%
%   AUC = AUC(TARGETS, OUTPUTS) where TARGETS and OUTPUTS are row vectors,
%   calculates (by applying trapezoidal numerical integration) the area AUC 
%   under the receiver operating characteristic curve of a classifier with
%   outputs OUTPUTS and targets TARGETS.
%
%   [AUC,TPR,FPR,THRESHOLDS] = AUC(TARGETS, OUTPUTS) also returns for each
%   threshold the true positive and false positive rate of the classifier
%   in row vectors TPR and FPR, while the corresponding thresholds are 
%   returned in the THRESHOLDS row vector.
%
%   AUC = AUC(TARGETS, OUTPUTS) where TARGETS and OUTPUTS are 1-by-N
%   (N-by-1) cell arrays of row vectors, calculates for each pair of 
%   vectors the area under the ROC curve and returns the areas in the
%   N-by-1 column vector AUC.
%
%   [AUC,TPR,FPR,THRESHOLDS] = AUC(TARGETS, OUTPUTS), where TARGETS and
%   OUTPUTS are 1-by-N (N-by-1) cell arrays of row vectors, also returns
%   the true positive and false positive rates in the 1-by-N (N-by-1) cell
%   arrays of row vectors TPR and FPR respectivelly, while the 
%   corresponding thresholds are returned in the 1-by-N (N-by-1) cell array
%   of row vectors THRESHOLDS. The areas are returned in the 1-by-N
%   (N-by-1) vector AUC.
%
%   See also ROC, ORG.MENSXMACHINA.COMMONS.STAT.ROC,
%       PLOTROC, ORG.MENSXMACHINA.COMMONS.STAT.PLOTROC

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

[tpr fpr thresholds] = ...
        org.mensxmachina.commons.stats.roc(targets, outputs);

if ~iscell(tpr)
    
    auc = trapz(fpr, tpr);
    
elseif iscell(tpr)

    auc = zeros(size(tpr));
    
    for i=1:length(tpr)
        auc(i) = trapz(fpr{i}, tpr{i});
    end
    
end

end