function [h p stats] = chi2test(a, x, y, varargin)
%CHI2TEST Chi(or G)-square test of conditional independence.
%   CHI2TEST performs a hypothesis test of conditional independence
%   for discrete distributions.  The test is performed by calculating the
%   observed and expected counts for each bin of the distribution, and
%   computing the chi-square test statistic SUM((O-E).^2./E) or G-square
%   test statistic 2*SUM(O.*log(O./E)), where O is the observed counts
%   and E is the expected counts under the null hypothesis that 
%   conditional independence holds. These test statistics have an 
%   approximate chi-square distribution when the counts are sufficiently
%   large.
% 
%   H = CHI2TEST(A, X, Y, S) performs a hypothesis test that the data in
%   column X of dataset A are independent of the data in column Y, given
%   the data in columns S. The result is H=0 if the null hypothesis (that
%   A(X) is independent of A(Y) given A(S)) cannot be rejected at the
%   5% significance level, or H=1 if the null hypothesis can be rejected at
%   the 5% level. A must a dataset with categorical only (nominal or
%   ordinal) variables.
%
%   CHI2TEST concludes that the null hypothesis cannot be rejected and does
%   not perform the test if the average observed count per cell (bin) is
%   smaller than OMIN=5.
%
%   CHI2TEST concludes that the null hypothesis cannot be rejected if the
%   degrees of freedom of the test are found out to be less than 1.
%
%   CHI2TEST treats undefined values in A as missing values, and ignores
%   them.
%
%   [...] = CHI2TEST(A, X, Y, S, 'Param1',VAL1,'Param2',VAL2,...) specifies
%   additional parameter name/value pairs chosen from the following:
%
%      'alpha'      A scalar from 0 to 1 that specifies the significance
%                   level for the hypothesis test as (100*ALPHA)%. 
%                   Default is 0.05.
%
%      'stat'       The test statistic to use. STAT can be either 'chi2' 
%                   for the Chi-square statistic or 'g2' for the G-square
%                   statistic.
%
%      'omin'       A non-negative integer that specifies the minimum
%                   average observed count per cell (bin) required for the 
%                   to be performed. CHI2TEST does not perform the test if
%                   the average observed count per cell is smaller than
%                   OMIN and concludes that the null hypothesis cannot be
%                   rejected (H=0).
%
%   [H,P] = CHI2TEST(...) also returns the p-value P.  The P value is the
%   probability of observing the observed statistic or a more extreme one
%   when the null hypothesis holds.
%
%   [H,P,STATS] = CHI2TEST(...) also returns a STATS structure with the
%   following fields:
%
%      'chi2stat'  Chi-square statistic if the test was performed,
%                  NaN otherwise, if STAT is 'chi2', or
%      'g2stat'    G-square statistic if the test was performed,
%                  NaN otherwise, if STAT is 'g2'
%      'df'        Degrees of freedom if the test was performed,
%                  NaN otherwise, 
%      'O'         Observed count in each bin if the test was performed,
%                  [] otherwise
%      'E'         Expected count in each bin if the test was performed,
%                  [] otherwise
%
%   See also STAT.CHI2TESTOPT

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.*;
import org.mensxmachina.commons.stats.*;

p = inputParser;  % Create an instance of the inputParser class
p.FunctionName = 'STATS.CHI2TEST';

p.addRequired('a', @(a) isa(a, 'dataset'));
p.addRequired('x', @(x) ischar(x) || (isscalar(x) && ispositiveinteger(x)));
p.addRequired('y', @(y) ischar(x) || (isscalar(x) && ispositiveinteger(x)));
p.addOptional('s', [], @(s) iscellstr(s) || (isvector(s) && all(ispositiveinteger(s))));

p.addParamValue('alpha', 0.05, @(alpha) isscalar(alpha) && alpha >= 0 && alpha <= 1);
p.addParamValue('stat', 'chi2', @(stat) any(strcmpi(stat, {'chi2', 'g2'})));
p.addParamValue('omin', 5, @(omin) isscalar(omin) && omin>=0 && isreal(omin) && omin==round(omin));
p.addParamValue('subs', [], @(subs) isnumeric(subs) && ndims(subs) == 2 );
p.addParamValue('nLevels', [], @(nLevels) isnlevels(a, nLevels) );

p.parse(a, x, y, varargin{:});

s = p.Results.s;
alpha = p.Results.alpha;
stat = p.Results.stat;

if strcmp(stat, 'chi2')
    stat = 1;
else
    stat = 2;
end

omin = p.Results.omin;

subs = p.Results.subs;

if isempty(subs)
    % convert dataset to subscripts
    subs = double(a);
end

nLevels = p.Results.nLevels;

if isempty(nLevels)
    % get # of levels
    nLevels = nlevels(a);
end

if nargout == 3 % struct of stats required
    
    % call optimized version of the function
    [h p t df O E] = ...
        chi2testopt(subs, nLevels, x, y, s, alpha, omin, stat, true);
    
    if stat == 1 % chi2stat
        stats = struct('chi2stat', t, 'df', df, 'O', O, 'E', E);
    else
        stats = struct('g2stat', t, 'df', df, 'O', O, 'E', E);
    end
    
else
    
    % call optimized version of the function
    [h p] = ...
        org.mensxmachina.ht.chi2testopt(subs, nLevels, x, y, s, alpha, omin, stat, true);
        
end

end