function [h p t df O E] = ...
    chi2testopt(subs, nLevels, x, y, s, alpha, omin, stat, removeNaNs)
%CHI2TEST optimized version of ORG.MENSXMAXHINA.STAT.CHI2TESTOPT
%
%   H = CHI2TESTOPT(SUBS, NLEVELS, X, Y, S) performs a hypothesis test
%   that the data in column X of matrix SUBS are independent of the data in
%   column Y, given the data in columns S. The result is H=0 if the null
%   hypothesis (that SUBS(X) is independent of SUBS(Y) given SUBS(S))
%   cannot be rejected at the 5% significance level, or H=1 if the null
%   hypothesis can be rejected at the 5% level. SUBS is a matrix of levels
%   and NLEVELS is the number of levels vector of SUBS. SUBS must not
%   contain unknown (NaN) values.
%
%   CHI2TESTOPT concludes that the null hypothesis cannot be rejected and 
%   does not perform the test if the average observed count per cell (bin) 
%   is smaller than OMIN=5.
%
%   CHI2TESTOPT concludes that the null hypothesis cannot be rejected if 
%   the degrees of freedom of the test are found out to be less than 1.
%
%   H = CHI2TESTOPT(SUBS, NLEVELS, X, Y, S, ALPHA) performs the test at the
%   significance level (100*ALPHA)%.  ALPHA must be a scalar from 0 to 1.
%
%   H = CHI2TESTOPT(SUBS, NLEVELS, X, Y, S, ALPHA, OMIN) does not perform
%   the test if the average observed count per cell (bin) is smaller than
%   OMIN and concludes that the null hypothesis cannot be rejected. OMIN
%   must be a non-negative integer.
%
%   H = CHI2TESTOPT(SUBS, NLEVELS, X, Y, S, ALPHA, OMIN, STAT) performs the
%   test at the significance level (100*ALPHA)% using the STAT statistic.
%   STAT must be 1 for the Chi-square statistic or 2 for the G-square
%   statistic.
%
%   H = CHI2TESTOPT(SUBS, NLEVELS, X, Y, S, ALPHA, OMIN, STAT, REMOVENANS)
%   removes unknown (NaN) values from SUBS before proceeding with the test.
%
%   [H,P] = CHI2TESTOPT(...) also returns the p-value P.  The P value is 
%   the probability of observing the observed statistic or a more extreme
%   one when the null hypothesis holds.
%
%   [H,P,T] = CHI2TEST(...) also returns the value T of the statistic if
%   the test  was performed, NaN otherwise.
%
%   [H,P,T,DF] = CHI2TEST(...) also returns the degrees of freedom DF if 
%   the test was performed, NaN otherwise.
%
%   [H,P,T,DF,O] = CHI2TEST(...) also returns the observed count O in each
%   bin if the test was performed, [] otherwise.
%
%   [H,P,T,DF,O,E] = CHI2TEST(...) also returns the expected count E in 
%   each bin if the test was performed, [] otherwise.
%
%   See also ORG.MENSXMACHINA.COMMONS.STAT.CHI2TEST

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

subs = subs(:, [x y s]);

if nargin < 6 || isempty(alpha)
    alpha = 0.05;
end

if nargin < 7 || isempty(omin)
    stat = 5;
end

if nargin < 8 || isempty(stat)
    stat = 1; % chi2stat
end

if nargin < 9 || isempty(removeNaNs)
    removeNaNs = false;
end

if removeNaNs
    % remove rows with NaNs in the variables of interest
    subs(any(isnan(subs), 2),:) = [];
end

[nobs ntestvars] = size(subs);

if nobs < omin*prod(nLevels([x y s]))
    
    % the average observed count per cell is smaller than OMIN
    %{
    warning('org:mensxmachina:ht:chi2test:LowCounts',...
            ['Some cells have observed counts less than OMIN'...
             '; not performing the test.']);
    %}
    % abort
    h = 0; % assuming independence (can't reject H0)
    p = NaN;
    t = NaN;
    df = NaN;
    O = [];
    E = [];
    
    % OK
    %disp('NAN p');
    %[p_ce, G2_ce, exit_flag]=g2test_2(x, y, s, data, nLevels);
    %assert( isnan(p_ce) );    
    
    return;
    
else
    
% compute observed level combination counts
Obs = accumarray(subs, ones(1, nobs), nLevels([x y s]));
    
% Obs_xs(x,y,s{:}): observed count of (x,s{:}), same for all y
ObsSum2 = sum(Obs, 2); % sum for all levels of y
Obs_xs_size = ones(1, ntestvars);
Obs_xs_size(2) = nLevels(y);
Obs_xs = repmat(ObsSum2, Obs_xs_size); % replicate for all levels of y

% Obs_ys(x,y,s{:}): observed count of (y,s{:}), same for all x
ObsSum1 = sum(Obs, 1); % sum for all levels of x
Obs_ys_size = ones(1, ntestvars);
Obs_ys_size(1) = nLevels(x);
Obs_ys = repmat(ObsSum1, Obs_ys_size); % replicate for all levels of x

% Obs_s(x,y,s{:}): observed count of (s{:}), same for all x and y
Obs_s = sum(ObsSum1, 2); % sum for all levels of x and y
Obs_s_size = ones(1, ntestvars);
Obs_s_size([1 2]) = nLevels([x y]);
Obs_s = repmat(Obs_s, Obs_s_size); % replicate for all levels of x and y

% compute expected level combination counts
Exp = Obs_xs.*Obs_ys./Obs_s;

if nargout >= 5 % O should be returned    
    O = Obs; % keep it at its current shape    
    if nargout == 6  % E should be returned
        E = Exp; % keep it at its current shape
    end
end

%Obs = reshape(Obs, nLevels(x), nLevels(y), []);
%df = sum( arrayfun(@(k) max(nLevels(x) - 1 - sum(~any(Obs(:,:,k), 2)), 0) * max(nLevels(y) - 1 - sum(~any(Obs(:,:,k), 1)), 0), 1:prod(nLevels(s))));

snlevelsprod = prod(nLevels(s));

ObsSum1 = reshape(ObsSum1, nLevels(y), snlevelsprod);
ObsSum2 = reshape(ObsSum2, nLevels(x), snlevelsprod);

% note: for seems to work faster
df = 0;
for k=1:snlevelsprod
    df = df + max(nLevels(x) - 1 - sum(~ObsSum2(:,k)), 0) * max(nLevels(y) - 1 - sum(~ObsSum1(:,k)), 0);
end

%{
df = 0;
sLevelCombinationInd = 1:prod(nLevels(s));
for k=sLevelCombinationInd
    df = df + max(nLevels(x) - 1 - sum(~any(Obs(:,:,k), 2)), 0) * max(nLevels(y) - 1 - sum(~any(Obs(:,:,k), 1)), 0);
end
%}

Obs = Obs(:);
Exp = Exp(:);

%df_removed = sum( Obs == 0 );

% figure out degrees of freedom
%df = (nLevels(x) - 1)*(nLevels(y) - 1)*prod(nLevels(s));

% is nansum OK?
% compute test statistic
if strcmp(stat, 'chi2stat')
    t = nansum((Obs-Exp).^2./Exp);
else % g2stat
    t = 2*nansum(Obs.*log(Obs./Exp));
end

if df > 0
	% call gammainc to avoid roundoff in computing upper tail probability
	p = gammainc(t/2, df/2, 'upper');  % p = 1 - chi2cdf(t,df);
else
	%disp('Degrees of freedom < 0.');
	df = 0;
	p = NaN(class(t));
end

% Determine if the actual significances exceeds the desired significance
h = cast(p <= alpha, class(p));

end