function [h varargout] = grpstatplot(a, varargin)
%GPRSTATPLOT Plot summary statistic by group
%
%	GPRSTATPLOT(X,GROUPVAR,VAR), when X is a matrix, plots the line defined
%   by the means, computed by group, of the column in X specified by the 
%   positive integer VAR and the grouping variable value. GROUPVAR is a
%   positive integer that specifies the grouping column in X that defines
%   the groups.
%
%	GPRSTATPLOT(DS,GROUPVAR,VAR), when DS is a dataset array, plots the
%   line defined by the means, computed by group, of the variable in DS
%   specified by VAR and the group index, if the grouping variable is a
%   cell array of strings / a categorical array or the grouping variable
%   value, if it is a numeric vector. GROUPVAR is either a positive integer
%   or a variable name that specifies the grouping variable in DS that
%   defines the groups. VAR can also be either a positive integer or a
%   variable name in DS. 
%
%	GPRSTATPLOT(A,LEGENDGROUPVAR,XGROUPVAR,VAR), when A is a matrix, plots
%   for each of the groups specified by LEGENDGROUPVAR the lines defined by
%   the means, computed by group as specified by XGROUPVAR, of the column 
%   in X specified by VAR and the group index. A legend will be placed on
%   the axes, containing the values of the grouping variable specified by 
%   LEGENDGROUPVAR.
%
%	GPRSTATPLOT(DS,LEGENDGROUPVAR,XGROUPVAR,VAR), when DS is a dataset
%   array, plots for each of the groups specified by LEGENDGROUPVAR the
%   lines defined by the means, computed by group as specified by
%   XGROUPVAR, of the variable in DS specified by VAR and the group index,
%   if XGROUPVAR specifies a cell array of strings / categorical array
%   or the grouping variable value, if XGROUPVAR specifies a numeric array.
%   A legend will be placed on the axes, containing the level labels of the
%   grouping variable specified by LEGENDGROUPVAR, if it is a categorical
%   variable, or the grouping variable values, if it is a cell array of
%   strings / numeric array.
%
%   GPRSTATPLOT(..., 'stat', STAT) plots the stat specified by STAT. STAT
%   can be either a single function handle or name. See GRPSTATS
%   documentation for the specifications of such a function. However,
%   in contrast to GRPSTATS, the function specified by STAT should only
%   return a value that has a single row, so it can be plotted.
%
%   GPRSTATPLOT(..., 'errorstat', ERRORSTAT) also draws error bars at 
%   the points of the line(s) defined by the statistics, computed by group,
%   and the group index or value. ERRORSTAT can be either a single function
%   handle or name. See GRPSTATS documentation for the specifications of
%   such a function. In contrast to GRPSTATS, the function specified by
%   ERRORSTAT should only return a value that has up to two rows. If the
%   function returns a single row E then each error bar is a distance of
%   E(i) above and below the line so that each bar is symmetric and 2*E(i)
%   long. If the function returns a value with two rows L and U, then each
%   error bar is L(i)+U(i) long specifying the lower and upper error bars.
%
%   variables in DS that define the groups, and is a positive integer, a
%   vector of two positive integers, the VarNames of a dataset variable, a cell
%   array containing one or two dataset variable VarNames, or a logical vector 
%   with at most two elements that are true. 
%
%	GPRSTATPLOT(DS,GROUPVAR,VAR), when DS is a dataset array, plots the
%   line defined by the means, computed by group, of the variable in DS
%   specified by VAR and the grouping variable in DS specified by GROUPVAR.
%
%   H = GPRSTATPLOT(...) returns a column vector of handles to lineseries
%   graphics objects, one handle per line (See PLOT documentation).
%
%	[H MEANS] = GPRSTATPLOT(X,GROUPVAR,VAR) and
%	[H MEANS] = GPRSTATPLOT(X,LEGENDGROUPVAR,GROUPVAR,VAR), when X is a
%   matrix, returns the means, computed by group, of the column in X 
%   specified by VAR, in the column vector MEANS.
%
%	[H STATS] = GPRSTATPLOT(X,GROUPVAR,VAR,'stat',STAT) and
%	[H STATS] = GPRSTATPLOT(X,LEGENDGROUPVAR,GROUPVAR,VAR,'stat',STAT), 
%   when X is a matrix, returns the statistics, computed by group with
%   STAT, of the column in X specified by VAR, in the column vector A.
%
%	[H STATS ERRORSTATS] = ...
%       GPRSTATPLOT(X,GROUPVAR,VAR,...,'errorstat',ERRORSTAT), and
%	[H STATS ERRORSTATS] = ...
%       GPRSTATPLOT(X,LEGENDGROUPVAR,GROUPVAR,VAR,
%           ...,'errorstat',ERRORSTAT), also returns
%   the errors of the statistics, computed by group with ERRORSTAT, of the 
%   column in X specified by VAR, in the column vector ERRORSTATS.
%
%	[H B] = GPRSTATPLOT(DS,GROUPVAR,VAR) and
%	[H B] = GPRSTATPLOT(DS,LEGENDGROUPVAR,GROUPVAR,VAR), when DS is a
%   dataset array, returns a dataset array B that contains the means, 
%   computed by group, of the variable in DS specified by VAR.
%
%	[H B] = GPRSTATPLOT(DS,GROUPVAR,VAR,'stat',STAT) and
%	[H B] = GPRSTATPLOT(DS,LEGENDGROUPVAR,GROUPVAR,VAR,'stat',STAT), 
%   when DS is a dataset array, returns a dataset array B that contains the
%   statistics, computed by group with STAT, of the variable in DS 
%   specified by VAR.
%
%	[H B] = GPRSTATPLOT(DS,GROUPVAR,VAR,...,'errorstat',ERRORSTAT), and
%	[H B] = ...
%       GPRSTATPLOT(DS,LEGENDGROUPVAR,GROUPVAR,VAR, ...
%           ...,'errorstat',ERRORSTAT), when DS is a dataset array, returns
%   a dataset array B that contains the statistics and their errors, 
%   computed by group, of the variables in DS specified by VAR.
%
%   [H B] = GPRSTATPLOT(DS,GROUPVAR,VAR,...,'VarNames',names), and
%	[H B] = ...
%       GPRSTATPLOT(DS,LEGENDGROUPVAR,GROUPVAR,VAR,...,'VarNames',names)
%   specifies the names of the variables in B (See GRPSTATS).
%
%   See also GRPSTATS, PLOT.

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

import org.mensxmachina.commons.*;

% detect two grouping variables mode
isTwoGroupVarsMode = rem(nargin, 2) == 0;

% parse input arguments with inputParser
p = inputParser;
p.FunctionName = 'ORG.MENSXMACHINA.COMMONS.STAT.GRPSTATPLOT';

p.addRequired('a', @(a) isacell(a, 'dataset') || ndims(a) == 2);

if isTwoGroupVarsMode
    
    p.addRequired('legendGroupVar', ...
        @(legendGroupVar) ischar(legendGroupVar) || (isscalar(legendGroupVar) && ispositiveinteger(legendGroupVar)) );

end

p.addRequired('xGroupVar', ...
    @(xGroupVar) ischar(xGroupVar) || (isscalar(xGroupVar) && ispositiveinteger(xGroupVar)) );

p.addRequired('var', ...
    @(var) ischar(var) || (isscalar(var) && ispositiveinteger(var)));

p.addParamValue('stat', ...
    @mean, @(stat) ischar(stat) || isa(stat, 'function_handle') );

p.addParamValue('errorstat', ...
    [], @(errorstat) isempty(errorstat) || isa(errorstat, 'function_handle') );

p.addParamValue('VarNames', [], @iscellstr);

p.parse(a, varargin{:});

if isTwoGroupVarsMode
    legendGroupVar = p.Results.legendGroupVar;
end

xGroupVar = p.Results.xGroupVar;
var = p.Results.var;
stat = p.Results.stat;
errorstat = p.Results.errorstat;
VarNames = p.Results.VarNames;

% detect other modes
isDatasetMode = isa(a, 'dataset');
isErrorBarMode = ~isempty(errorstat);

if isDatasetMode

    % get input dataset varnames
    inputVarNames = get(a, 'VarNames');
    
    % get X grouping variable name
    if ischar(xGroupVar)
        xGroupVarName = xGroupVar;
    else
        xGroupVarName = inputVarNames{xGroupVar};
    end
    
    % detect X grouping variable state
    if isa(a.(xGroupVarName), 'categorical') % categorical
        xGroupVariableState = 'categorical';
    elseif iscellstr(a.(xGroupVarName)) % cell array of strings
        xGroupVariableState = 'cellstr';
    else
        xGroupVariableState = 'numeric';
    end

    % get X grouping variable observed values
    xGroupVarObservedValues = unique(a.(xGroupVarName));
    
    % get X grouping variable obserbed labels
    if strcmp(xGroupVariableState, 'categorical')
        xGroupVarObservedLabels = cellstr(xGroupVarObservedValues);
    elseif strcmp(xGroupVariableState, 'cellstr')
        xGroupVarObservedLabels = xGroupVarObservedValues';
    else % numeric
        xGroupVarObservedLabels = arrayfun( ...
            @num2str, xGroupVarObservedValues, 'UniformOutput', false)';
    end
    
    if isTwoGroupVarsMode 
        
        % get legend grouping variable name
        if ischar(legendGroupVar)
            legendGroupVarName = legendGroupVar;
        else
            legendGroupVarName = inputVarNames{legendGroupVar};
        end
        
        % detect legend grouping variable state
        if isa(a.(legendGroupVarName), 'categorical') % categorical
            legendGroupVariableState = 'categorical';
        elseif iscellstr(a.(legendGroupVarName)) % cell array of strings
            legendGroupVariableState = 'cellstr';
        else
            legendGroupVariableState = 'numeric';
        end

        % get legend grouping variable observed values
        legendGroupVarObservedValues = unique(a.(legendGroupVarName));

        % get legend grouping variable obserbed labels
        if strcmp(legendGroupVariableState, 'categorical')
            legendGroupVarObservedLabels = cellstr(legendGroupVarObservedValues);
        elseif strcmp(legendGroupVariableState, 'cellstr')
            legendGroupVarObservedLabels = legendGroupVarObservedValues';
        else % numeric
            legendGroupVarObservedLabels = arrayfun( ...
                @num2str, legendGroupVarObservedValues, 'UniformOutput', false)';
        end       
        
    end
    
    % create groupvars input argument for grpstats
    if ~isTwoGroupVarsMode
        groupvars = xGroupVarName;
    else
        groupvars = {legendGroupVarName, xGroupVarName};
    end
    
    % create whichstats input argument for grpstats
    if ~isErrorBarMode
        whichstats = stat;
    else
        whichstats = {stat, errorstat};
    end
    
    % create param-value input arguments for grpstats
    if ~isempty(VarNames)
        params = {'DataVars', var, 'VarNames', VarNames};
    else
        params = {'DataVars', var};     
    end
    
    % calculate group statistics   
    varargout{1} = grpstats(a, groupvars, whichstats, params{:});
   
    % get output dataset variable VarNames
    outputVarNames = get(varargout{1}, 'VarNames');
    
    % string columns come unsorted; sort them now
    
    if ~isTwoGroupVarsMode && strcmp(xGroupVariableState, 'cellstr')
        
        varargout{1} = sortrows(varargout{1}, outputVarNames{1});
        
    elseif isTwoGroupVarsMode
        
        if strcmp(legendGroupVariableState, 'cellstr') % 1st is string
            
            if ~strcmp(xGroupVariableState, 'cellstr') % 2nd is not            
                varargout{1} = sortrows(varargout{1}, outputVarNames{1});
            else % 2nd is too
                varargout{1} = sortrows(varargout{1}, outputVarNames(1:2));
            end
            
        elseif strcmp(xGroupVariableState, 'cellstr') % 1st is not, 2nd is
            
            for i=1:numel(legendGroupVarObservedValues)
                % for each legend group variable value
                
                % find indices of rows corresponding to this value
                rows = varargout{1}.(outputVarNames{1}) == legendGroupVarObservedValues(i);
                
                % sort those rows by x group variable
                varargout{1}(rows, :) = sortrows(varargout{1}(rows, :), outputVarNames{2});
                
            end
            
        end
        
    end
    
    % get stat variable name
    if ~isTwoGroupVarsMode
        statVarName = outputVarNames{3};
    else
        statVarName = outputVarNames{4};
    end    
    
    % get number of observed values of X grouping variable
    n = numel(xGroupVarObservedValues);
    
    % create X vector for the plot function
    if strcmp(xGroupVariableState, 'categorical')
        x = (1:n)';
    elseif strcmp(xGroupVariableState, 'cellstr')
        x = (1:n)';
    else % numeric
        x = xGroupVarObservedValues; % (already a column vector)
    end
    
    if isTwoGroupVarsMode
        % get legend grouping variable observed values
        legendGroupVarObservedValues = unique(a.(legendGroupVarName));
        % get number of observed values of legend grouping variable
        m = numel(legendGroupVarObservedValues);
    end
    
    % create Y vector/matrix for the plot/errorbar function
    if ~isTwoGroupVarsMode
        y = double(varargout{1}(:, 3));
    else
        m = numel(legendGroupVarObservedValues);
        y = reshape(double(varargout{1}(:, 4)), n, m);
    end
    
    if ~isErrorBarMode % plot
        
        h = plot(x, y);

        % X ticks
        set(gca,'XTick', x); % ticks on group ids
        
    else % errorbar
        
        if ~isTwoGroupVarsMode
            e = double(varargout{1}(:, 4));
            numVals = size(e, 2);
            if numVals == 1
                h = errorbar(x, y, e);
            else
                l = e(:, 1);
                u = e(:, 2);
                h = errorbar(x, y, l, u);
            end
        else
            % replicate X to a matrix (needed for errorbar)
            x = repmat(x, 1, m);
            e = double(varargout{1}(:, 5));
            numVals = size(e, 2);
            if numVals == 1
                e = reshape(e, n, m);
                h = errorbar(x, y, e);
            else
                l = reshape(e(:, 1), n, m);
                u = reshape(e(:, 2), n, m);
                h = errorbar(x, y, l, u);
            end
            
        end        

        % X ticks
        set(gca,'XTick', x(:, 1)); % ticks on group ids
        
    end
 
    % label X axis
    xlabel(xGroupVarName);
    
    % label Y axiss
    ylabel(statVarName);
    
    % set X axis labels
    set(gca,'XTickLabel', xGroupVarObservedLabels);
    
    % set title
    title(sprintf('%s vs %s', statVarName, xGroupVarName));
    
    if isTwoGroupVarsMode
        % set legend
        legend(legendGroupVarObservedLabels);
    end
    
else % matrix mode
    
    % create group input argument for grpstats
    if ~isTwoGroupVarsMode
        group = a(:, xGroupVar);
    else
        group = a(:, [legendGroupVar xGroupVar]);
    end    
    
    % calculate group statistics
    if ~isErrorBarMode
        varargout{1} = grpstats(a(:, var), group, stat);
    else
        [varargout{1} varargout{2}] = ...
            grpstats(a(:, var), group, {stat, errorstat});
    end
    
    xGroupVarObservedValues = unique(a(:, xGroupVar)); % column vector / c. array
    n = numel(xGroupVarObservedValues);
    x = xGroupVarObservedValues;
    
    if isTwoGroupVarsMode
        legendGroupVarObservedValues = unique(a(:, legendGroupVar));
        m = numel(legendGroupVarObservedValues);
    end
    
    % create Y vector/matrix for the plot/errorbar function
    if ~isTwoGroupVarsMode
        y = varargout{1};
    else
        m = numel(legendGroupVarObservedValues);
        y = reshape(varargout{1}, n, m);
    end
    
    if ~isErrorBarMode
        
        h = plot(x, y);

        % X ticks
        set(gca,'XTick', x); % ticks on group ids
        
    else

        numVals = size(varargout{2}, 2);
        
        if ~isTwoGroupVarsMode
            if numVals == 1
                e = varargout{2};
                h = errorbar(x, y, e);
            else
                l = varargout{2}(:, 1);
                u = varargout{2}(:, 2);
                h = errorbar(x, y, l, u);
            end
        else
            % replicate X to a matrix (needed for errorbar)
            x = repmat(x, 1, m);
            if numVals == 1
                e = reshape(varargout{2}, n, m);
                h = errorbar(x, y, e);
            else
                l = reshape(varargout{2}(:, 1), n, m);
                u = reshape(varargout{2}(:, 2), n, m);
                h = errorbar(x, y, l, u);
            end
            
        end        

        % X ticks
        set(gca,'XTick', x(:, 1)); % ticks on group ids
        
    end
    
end

end