function [tpr fpr threshold] = roc(targets, outputs)
%ROC Receiver operating characteristic
%
%   [TPR,FPR,THRESHOLDS] = ROC(TARGETS, OUTPUTS) where TARGETS and OUTPUTS
%   are row (column) vectors of the same size, calculates for each
%   threshold the true positive and false positive rate of a classifier 
%   with outputs OUTPUTS and targets TARGETS and returns them in row
%   (column) vectors TPR and FPR, while the corresponding threshold are 
%   returned in the THRESHOLDS row (column) vector. Each pair of values
%   (TPR(k), FPR(k)) defines a point in the Receiver operating
%   characteristic (ROC) curve.
%
%   [TPR,FPR,THRESHOLDS] = ROC(TARGETS, OUTPUTS) where TARGETS and OUTPUTS
%   are 1-by-N (N-by-1) cell arrays of vectors, calculates for each pair of
%   vectors the positive and false positive rates and returns them in the
%   1-by-N (N-by-1) cell arrays of vectors TPR and FPR respectivelly, while
%   the corresponding threshold are returned in the 1-by-N (N-by-1) cell 
%   array of vectors THRESHOLDS.
%
%   See also:
%       AUC, ROC, ORG.MENSXMACHINA.COMMONS.STAT.ROC,
%       PLOTROC, ORG.MENSXMACHINA.COMMONS.STAT.PLOTROC,

%   Copyright 2010 Mens X Machina
%
%   This file is part of Mens X Machina Commons Toolbox.
%
%   Mens X Machina Commons Toolbox is free software: you can redistribute
%   it and/or modify it under the terms of the GNU General Public License 
%   as published by the Free Software Foundation, either version 3 of the
%   License, or (at your option) any later version.
%
%   Mens X Machina Commons Toolbox is distributed in the hope that it will
%   be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
%   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
%   General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with Mens X Machina Commons Toolbox. If not, see
%   <http://www.gnu.org/licenses/>.

if ~iscell(targets)
    
    [tpr fpr threshold] = roc_single(targets, outputs);

else
    
    tpr = cell(size(targets));
    fpr = cell(size(targets));
    threshold = cell(size(targets));
    
    for i=1:length(targets)
        [tpr{i} fpr{i} threshold{i}] = roc_single(targets{i}, outputs{i});
    end

end

end

function [tpr fpr threshold] = roc_single(targets, outputs)
%ROC_SINGLE Single receiver operating characteristic

p = sum(targets); % # of positives
n = numel(targets) - p; % # of negatives

% sort outputs and targets by score
[outputs sortInd] = sort(outputs, 'descend');
targets = targets(sortInd);

fp = 0;
tp = 0;

if size(targets, 1) == 1 % row vector
    threshold = [ -Inf unique(outputs) Inf ];
else
    threshold = [ -Inf; unique(outputs); Inf ];
end

fpr = zeros(size(threshold));
tpr = zeros(size(threshold));

f_prev = -Inf;

pointCount = 1;

for i=1:numel(targets) % for each example
    
    if outputs(i) ~= f_prev
        pointCount = pointCount + 1;
        fpr(pointCount) = fp/n;
        tpr(pointCount) = tp/p;
        f_prev = outputs(i);
    end
    
    if targets(i) % i is a positive example
        tp = tp + 1;
    else % i is a negative example
        fp = fp + 1;
    end
    
end

% this is (1,1)
fpr(end) = fp/n;
tpr(end) = tp/p;

end